@@ -34,6 +34,7 @@ from dopamine.jax import losses
 from dopamine.jax import networks
 from dopamine.jax.agents.dqn import dqn_agent
 from dopamine.jax.agents.rainbow import rainbow_agent
+from dopamine.metrics import statistics_instance
 from dopamine.replay_memory import prioritized_replay_buffer
 import gin
 import jax
@@ -302,11 +303,16 @@ class JaxFullRainbowAgent(dqn_agent.JaxDQNAgent):
                                 jnp.sqrt(loss + 1e-10))
 
     if self.summary_writer is not None:
-      summary = tf.compat.v1.Summary(value=[
-          tf.compat.v1.Summary.Value(
-              tag='CrossEntropyLoss', simple_value=mean_loss)
-      ])
-      self.summary_writer.add_summary(summary, self.training_steps)
+      with self.summary_writer.as_default():
+        tf.summary.scalar('CrossEntropyLoss', mean_loss,
+                          step=self.training_steps)
+      self.summary_writer.flush()
+      if hasattr(self, 'collector_dispatcher'):
+        self.collector_dispatcher.write(
+            [statistics_instance.StatisticsInstance(
+                'Loss', mean_loss.to_py(), step=self.training_steps),
+             ],
+            collector_allowlist=self._collector_allowlist)
 
   def _store_transition(self,
                         last_observation,
